==> nautilus_trader-1.205.0.tar.gz: nautilus_trader-1.205.0/nautilus_core/core/src/python/uuid.rs <==
// -------------------------------------------------------------------------------------------------
//  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
//  https://nautechsystems.io
//
//  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
//  You may not use this file except in compliance with the License.
//  You may obtain a copy of the License at https://www.gnu.org/licenses/lgpl-3.0.en.html
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
// -------------------------------------------------------------------------------------------------

use std::{
    collections::hash_map::DefaultHasher,
    hash::{Hash, Hasher},
    str::FromStr,
};

use pyo3::{
    prelude::*,
    pyclass::CompareOp,
    types::{PyBytes, PyTuple},
};

use super::to_pyvalue_err;
use crate::uuid::{UUID4, UUID4_LEN};

#[pymethods]
impl UUID4 {
    /// Creates a new [`UUID4`] instance.
    ///
    /// If a string value is provided, it attempts to parse it into a UUID.
    /// If no value is provided, a new random UUID is generated.
    #[new]
    #[pyo3(signature = (value=None))]
    fn py_new(value: Option<&str>) -> PyResult<Self> {
        match value {
            Some(val) => Self::from_str(val).map_err(to_pyvalue_err),
            None => Ok(Self::new()),
        }
    }

    /// Sets the state of the `UUID4` instance during unpickling.
    fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
        let bytes: &Bound<'_, PyBytes> = state.downcast_bound::<PyBytes>(py)?;
        let slice = bytes.as_bytes();

        if slice.len() != UUID4_LEN {
            return Err(to_pyvalue_err(
                "Invalid state for deserialzing, incorrect bytes length",
            ));
        }

        self.value.copy_from_slice(slice);
        Ok(())
    }

    /// Gets the state of the `UUID4` instance for pickling.
    fn __getstate__(&self, _py: Python) -> PyResult<PyObject> {
        Ok(PyBytes::new_bound(_py, &self.value).to_object(_py))
    }

    /// Reduces the `UUID4` instance for pickling.
    fn __reduce__(&self, py: Python) -> PyResult<PyObject> {
        let safe_constructor = py.get_type_bound::<Self>().getattr("_safe_constructor")?;
        let state = self.__getstate__(py)?;
        Ok((safe_constructor, PyTuple::empty_bound(py), state).to_object(py))
    }

    /// A safe constructor used during unpickling to ensure the correct initialization of `UUID4`.
    #[staticmethod]
    fn _safe_constructor() -> PyResult<Self> {
        Ok(Self::new()) // Safe default
    }

    /// Compares two `UUID4` instances for equality
    fn __richcmp__(&self, other: &Self, op: CompareOp, py: Python<'_>) -> Py<PyAny> {
        match op {
            CompareOp::Eq => self.eq(other).into_py(py),
            CompareOp::Ne => self.ne(other).into_py(py),
            _ => py.NotImplemented(),
        }
    }

    /// Returns a hash value for the `UUID4` instance.
    fn __hash__(&self) -> isize {
        let mut h = DefaultHasher::new();
        self.hash(&mut h);
        h.finish() as isize
    }

    /// Returns a detailed string representation of the `UUID4` instance.
    fn __repr__(&self) -> String {
        format!("{self:?}")
    }

    /// Returns the `UUID4` as a string.
    fn __str__(&self) -> String {
        self.to_string()
    }

    /// Gets the `UUID4` value as a string.
    #[getter]
    #[pyo3(name = "value")]
    fn py_value(&self) -> String {
        self.to_string()
    }

    /// Creates a new `UUID4` from a string representation.
    #[staticmethod]
    #[pyo3(name = "from_str")]
    fn py_from_str(value: &str) -> PyResult<Self> {
        Self::from_str(value).map_err(to_pyvalue_err)
    }
}

==> nautilus_trader-1.206.0.tar.gz: nautilus_trader-1.206.0/nautilus_core/core/src/python/uuid.rs <==
// -------------------------------------------------------------------------------------------------
//  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
//  https://nautechsystems.io
//
//  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
//  You may not use this file except in compliance with the License.
//  You may obtain a copy of the License at https://www.gnu.org/licenses/lgpl-3.0.en.html
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
// -------------------------------------------------------------------------------------------------

use std::{
    collections::hash_map::DefaultHasher,
    hash::{Hash, Hasher},
    str::FromStr,
};

use pyo3::{
    prelude::*,
    pyclass::CompareOp,
    types::{PyBytes, PyTuple},
};

use super::to_pyvalue_err;
use crate::uuid::{UUID4, UUID4_LEN};

#[pymethods]
impl UUID4 {
    /// Creates a new [`UUID4`] instance.
    ///
    /// If a string value is provided, it attempts to parse it into a UUID.
    /// If no value is provided, a new random UUID is generated.
    #[new]
    #[pyo3(signature = (value=None))]
    fn py_new(value: Option<&str>) -> PyResult<Self> {
        match value {
            Some(val) => Self::from_str(val).map_err(to_pyvalue_err),
            None => Ok(Self::new()),
        }
    }

    /// Sets the state of the `UUID4` instance during unpickling.
    fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
        let bytes: &Bound<'_, PyBytes> = state.downcast_bound::<PyBytes>(py)?;
        let slice = bytes.as_bytes();

        if slice.len() != UUID4_LEN {
            return Err(to_pyvalue_err(
                "Invalid state for deserialzing, incorrect bytes length",
            ));
        }

        self.value.copy_from_slice(slice);
        Ok(())
    }

    /// Gets the state of the `UUID4` instance for pickling.
    fn __getstate__(&self, py: Python) -> PyResult<PyObject> {
        Ok(PyBytes::new_bound(py, &self.value).to_object(py))
    }

    /// Reduces the `UUID4` instance for pickling.
    fn __reduce__(&self, py: Python) -> PyResult<PyObject> {
        let safe_constructor = py.get_type_bound::<Self>().getattr("_safe_constructor")?;
        let state = self.__getstate__(py)?;
        Ok((safe_constructor, PyTuple::empty_bound(py), state).to_object(py))
    }

    /// A safe constructor used during unpickling to ensure the correct initialization of `UUID4`.
    #[staticmethod]
    fn _safe_constructor() -> PyResult<Self> {
        Ok(Self::new()) // Safe default
    }

    /// Compares two `UUID4` instances for equality
    fn __richcmp__(&self, other: &Self, op: CompareOp, py: Python<'_>) -> Py<PyAny> {
        match op {
            CompareOp::Eq => self.eq(other).into_py(py),
            CompareOp::Ne => self.ne(other).into_py(py),
            _ => py.NotImplemented(),
        }
    }

    /// Returns a hash value for the `UUID4` instance.
    fn __hash__(&self) -> isize {
        let mut h = DefaultHasher::new();
        self.hash(&mut h);
        h.finish() as isize
    }

    /// Returns a detailed string representation of the `UUID4` instance.
    fn __repr__(&self) -> String {
        format!("{self:?}")
    }

    /// Returns the `UUID4` as a string.
    fn __str__(&self) -> String {
        self.to_string()
    }

    /// Gets the `UUID4` value as a string.
    #[getter]
    #[pyo3(name = "value")]
    fn py_value(&self) -> String {
        self.to_string()
    }

    /// Creates a new `UUID4` from a string representation.
    #[staticmethod]
    #[pyo3(name = "from_str")]
    fn py_from_str(value: &str) -> PyResult<Self> {
        Self::from_str(value).map_err(to_pyvalue_err)
    }
}
